import os
import sys
import warnings as _warnings

try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path  # type: ignore

import click  # type: ignore
import IPython  # type: ignore

from .config import build_config, load_rc
from .magic import *  # noqa
from .virtualenv import load_venv

_PYTHON_VERSION = "python{0}.{1}".format(*sys.version_info[0:2])
_HISS_CONFIG = Path(os.environ.get("HOME", "~"), ".hiss").expanduser()
_BANNER = """
hiss - {python_version}

"""


@click.command()
@click.option("--config", "-c", default=_HISS_CONFIG)
@click.option("--warnings/--no-warnings", default=False)
def main(config, warnings):
    # type: (str, bool) -> None
    """Console script for hiss"""
    banner = _BANNER.format(python_version=_PYTHON_VERSION)

    if not warnings:
        for warning in (UserWarning, DeprecationWarning, RuntimeWarning):
            _warnings.filterwarnings("ignore", category=warning)

    # check for and (optionally) enter virtualenv
    load_venv(_PYTHON_VERSION)

    rc = {}
    config = Path(config)
    if config.exists():  # type: ignore
        rc.update(load_rc(config))

    # build configuration
    hc = build_config(rc, banner)

    # start customized ipython!
    IPython.start_ipython(argv=[], config=hc, quick=True, auto_create=False)



import configparser
import io
import os

from typing import Any, Dict, Union, Tuple, Optional  # noqa

try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path  # type: ignore

from IPython.terminal.prompts import ClassicPrompts  # type: ignore

from traitlets.config import Config as Config  # type: ignore
from pygments.styles import get_style_by_name  # type: ignore
from pygments.util import ClassNotFound  # type: ignore

assert Path  # to shush flake8


def _bool(value):
    if not isinstance(value, bool):
        return str(value).lower() in ("true", "t", "1")
    return value


def load_rc(path):
    # type: (Path) -> Dict[str, str]
    """This function loads the .hiss file represented as a dict"""
    config = io.BytesIO()
    config.write(b"[hiss]\n")

    # attempt load of rc file
    try:
        with open(str(path)) as f:
            config.write(f.read().replace("%", "%%").encode("utf8"))
    except IOError:
        # no rc file found
        pass

    config.seek(0, os.SEEK_SET)
    parser = configparser.SafeConfigParser()
    parser.read_string(config.read().decode())

    return {key: str(value) for key, value in parser.items("hiss")}


def get_theme(rc):
    # type: (Dict[str, str]) -> Tuple[str, Optional[Any]]
    pygments_style = None
    theme = rc.get("theme", "legacy")

    if theme != "legacy":
        try:
            pygments_style = get_style_by_name(theme)
        except ClassNotFound:
            theme = "legacy"

    return theme, pygments_style


def build_config(rc, banner):
    # type: (Dict[str, str], str) -> Config

    rc = rc

    # initialize traitlets config obj
    config = Config()

    # hardcoded configs
    config.TerminalInteractiveShell.prompts_class = ClassicPrompts
    config.TerminalInteractiveShell.separate_in = ""
    config.TerminalInteractiveShell.true_color = True
    config.TerminalInteractiveShell.banner1 = banner
    config.PrefilterManager.multi_line_specials = True

    # overrideable configs
    config.TerminalInteractiveShell.confirm_exit = _bool(rc.get("confirm_exit", False))
    config.TerminalInteractiveShell.autoindent = _bool(rc.get("autoindent", True))
    config.TerminalInteractiveShell.colors = rc.get("colors", "Linux")
    config.TerminalInteractiveShell.editing_mode = rc.get("editing_mode", "emacs")

    # auto reload
    if _bool(rc.get("autoreload", False)):
        config.InteractiveShellApp.extensions = ["autoreload"]
        config.InteractiveShellApp.exec_lines = ["%autoreload 2"]

    # themes
    theme, pygments_style = get_theme(rc)
    config.TerminalInteractiveShell.highlighting_style = theme
    if pygments_style is not None:
        config.TerminalInteractiveShell.highlighting_style_overrides = (
            pygments_style.styles
        )

    # ensure magics will be loaded
    config.InteractiveShellApp.exec_lines.extend(
        [
            "from hiss.cli.magic import HissMagics",
            "get_ipython().register_magics(HissMagics)",
        ]
    )

    return config



import json
import os
import site
import sys
import zipfile

from imp import reload

try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path  # type: ignore

from IPython.core.magic import Magics, magics_class, line_magic  # type: ignore
from IPython import start_ipython  # type: ignore


@magics_class
class HissMagics(Magics):

    @line_magic
    def shiv(self, shiv_path):
        if shiv_path:
            sys.path.insert(0, shiv_path)
            from _bootstrap import cache_path  # type: ignore

            zf = zipfile.ZipFile(shiv_path)
            env = json.loads(zf.read("environment.json"))
            site.addsitedir(
                cache_path(zf, Path.home() / ".shiv", env["build_id"]) / "site-packages"
            )
            print("Bootstrapped into shiv {shiv_path}".format(shiv_path=shiv_path))
        else:
            print("No pyz provided! Doing nothing.")

    @line_magic
    def pex(self, entry_point):
        if entry_point:
            sys.path[0] = os.path.abspath(sys.path[0])
            sys.path.insert(0, entry_point)
            sys.path.insert(0, os.path.abspath(os.path.join(entry_point, ".bootstrap")))
            from _pex import pex_bootstrapper  # type: ignore

            pex_bootstrapper.bootstrap_pex_env(entry_point)
            self.reload_pkg_resources("")
            print("Bootstrapped into pex {0}.".format(entry_point))
        else:
            print("No pex provided! Doing nothing.")

    @line_magic
    def reload_pkg_resources(self, line):
        if "pkg_resources" in sys.modules:
            import pkg_resources

            reload(pkg_resources)

    @line_magic
    def debug(self, line):
        import logging

        logging.basicConfig(level=logging.DEBUG)
        log = logging.getLogger(__name__)  # noqa: F841
        print("Debug logging enabled.")

    @line_magic
    def flask(self, app):
        """This is hella sketchy"""
        try:
            # we assume we have access to flask/werkzeug,
            # if not, go ahead and raise the importerror
            from flask import _request_ctx_stack  # type: ignore
            from werkzeug import import_string  # type: ignore
        except ImportError as e:
            raise ImportError("You don't have flask/werkzeug! {err}".format(err=e))
        else:
            app = import_string(app)
            with app.test_request_context():
                banner = (
                    "\nNow using flask context from: {app_repr}\n"
                    "There is no escape! Exit the session when you are done.\n".format(
                        app_repr=repr(app)
                    )
                )

                # add our custom message
                get_ipython().banner1 = banner  # noqa: F821

                # starting a new session dorks up history, so end our sesh
                get_ipython().history_manager.end_session()  # noqa: F821
                get_ipython().history_manager.reset()  # noqa: F821

                # respawn ipython with the flask context
                start_ipython(user_ns=dict(app=_request_ctx_stack.top.app))



import os
import site

try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path  # type: ignore

from .magic import HissMagics


def load_venv(python_version):
    # type: (str) -> None
    if "VIRTUAL_ENV" in os.environ:
        virtual_env = Path(
            os.environ.get("VIRTUAL_ENV"), "lib", python_version, "site-packages"
        )
        if virtual_env.exists():
            site.addsitedir(str(virtual_env))
            HissMagics().reload_pkg_resources("")
            print("\x1B[3mvirtualenv detected -> {}\x1B[23m".format(virtual_env))



import os
import sys
import setuptools
import subprocess
import re

from setuptools.command import easy_install

install_requires = [
    "Click>=6.6",
    "cached_property",
    "traitlets",
    "configparser",
    "pygments",
    "six",
]

_py2_extras = ["IPython<6", "pathlib2", "typing"]

extras_require = {
    ":python_version<'3'": _py2_extras,
    ":python_version>='3'": ["IPython==6.5.0"],
}

if int(setuptools.__version__.split('.')[0]) < 18:
    extras_require = {}
    if sys.version_info < (3, 0):
        install_requires.extend(_py2_extras)
    else:
        install_requires.append("IPython==6.5.0")


# fast entry points, Copyright (c) 2016, Aaron Christianson
# https://github.com/ninjaaron/fast-entry_point
TEMPLATE = """\
# -*- coding: utf-8 -*-
# EASY-INSTALL-ENTRY-SCRIPT: '{3}','{4}','{5}'
__requires__ = '{3}'
import re
import sys
from {0} import {1}
if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    sys.exit({2}())"""


@classmethod
def get_args(cls, dist, header=None):
    """
    Yield write_script() argument tuples for a distribution's
    console_scripts and gui_scripts entry points.
    """
    if header is None:
        header = cls.get_header()
    spec = str(dist.as_requirement())
    for type_ in "console", "gui":
        group = type_ + "_scripts"
        for name, ep in dist.get_entry_map(group).items():
            # ensure_safe_name
            if re.search(r"[\\/]", name):
                raise ValueError("Path separators not allowed in script names")
            script_text = TEMPLATE.format(
                ep.module_name, ep.attrs[0], ".".join(ep.attrs), spec, group, name
            )
            args = cls._get_script_args(type_, name, header, script_text)
            for res in args:
                yield res


easy_install.ScriptWriter.get_args = get_args


class Venv(setuptools.Command):
    user_options = []

    def initialize_options(self):
        """Abstract method that is required to be overwritten"""

    def finalize_options(self):
        """Abstract method that is required to be overwritten"""

    def run(self):
        import venv

        venv_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), "venv", "hiss"
        )
        print("Creating virtual environment in {}".format(venv_path))
        venv.main(args=[venv_path])
        print(
            "Linking `activate` to top level of project.\n"
            "To activate, simply run `source activate`."
        )
        try:
            os.symlink(
                os.path.join(venv_path, "bin", "activate"),
                os.path.join(os.path.dirname(os.path.abspath(__file__)), "activate"),
            )
        except OSError:
            # symlink already exists
            pass


setuptools.setup(
    name="hiss-repl",
    version="3.1.3",
    description="A simple and easily configured iPython-based python repl",
    author="Loren Carvalho",
    author_email="me@loren.pizza",
    url="https://github.com/sixninetynine/hiss",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    entry_points={"console_scripts": ["hiss=hiss.cli:main"]},
    include_package_data=True,
    install_requires=install_requires,
    extras_require=extras_require,
    license="MIT license",
    keywords="hiss",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    cmdclass={"venv": Venv},
)
